from mmft.simulator import *

# Continuous Hybrid
def hybridConcentration():

    network = createNetwork()

    # Nodes
    n0 = network.addNode(0.0, 0.0, True)
    n1 = network.addNode(1e-3, 2e-3, False)
    n2 = network.addNode(1e-3, 1e-3, False)
    n3 = network.addNode(1e-3, 0.0, False)
    n4 = network.addNode(2e-3, 2e-3, False)
    n5 = network.addNode(1.75e-3, 1e-3, False)
    n6 = network.addNode(2e-3, 0.0, False)
    n7 = network.addNode(2e-3, 1.25e-3, False)
    n8 = network.addNode(2e-3, 0.75e-3, False)
    n9 = network.addNode(2.25e-3, 1e-3, False)
    n10 = network.addNode(3e-3, 1e-3, True)

    # Channels
    c0 = network.addRectangularChannel(n0, n1, 1e-4, 1e-4, ChannelType.normal)
    c1 = network.addRectangularChannel(n0, n2, 1e-4, 1e-4, ChannelType.normal)
    c2 = network.addRectangularChannel(n0, n3, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n1, n4, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n2, n5, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n3, n6, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n4, n7, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n6, n8, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n9, n10, 1e-4, 1e-4, ChannelType.normal)

    # Openings
    Opening1 = Opening(n5, [1.0, 0.0], 1e-4)
    Opening2 = Opening(n7, [0.0, -1.0], 1e-4)
    Opening3 = Opening(n8, [0.0, 1.0], 1e-4)
    Opening4 = Opening(n9, [-1.0, 0.0], 1e-4);

    # Module
    position = [1.75e-3, 0.75e-3]
    size = [5e-4, 5e-4]
    stlFile = "../examples/STL/cross.stl"
    openings = {n5.getId(): Opening1, 
                n7.getId(): Opening2, 
                n8.getId(): Opening3, 
                n9.getId(): Opening4}
    m0 = network.addCfdModule(position, size, stlFile, openings)

    # Pressure Pumps
    network.addPressurePump(n0, n1, 1e3)
    network.addPressurePump(n0, n2, 1e3)
    network.addPressurePump(n0, n3, 1e3)

    # Simulation
    simulation = HybridConcentration(network)

    # Fluid & Resistance Model
    f0 = simulation.addFluid(1e-3, 1e3)
    simulation.setContinuousPhase(f0)
    simulation.setPoiseuilleResistanceModel()

    simulation.setDiffusiveMixingModel()

    s1 = simulation.addSpecie(1e-9, 2.0, 1.0)

    m1 = simulation.addMixture(s1, 2.0)
    m2 = simulation.addMixture(s1, 1.0)
    simulation.addMixtureInjection(m1, c0, 0.0, 1.0)
    simulation.addMixtureInjection(m2, c1, 0.0, 1.0)
    simulation.addMixtureInjection(m2, c2, 0.0, 1.0)

    simulation.addLbmSimulator(m0, 20, 1e-1, 0.55, 0.55, 1e-4, 1e-1, "1a")

    simulation.setNaiveHybridScheme(0.1, 0.5, 10)

    simulation.simulate()

    # Results
    result = simulation.getResults()
    result.printLastState()

def main():
    hybridConcentration()

if __name__ == "__main__":
    main()


from pysimulator import *

# Continuous Abstract
def abstractContinuous():

    # Define the network
    network = Network()

    # Nodes
    n0 = network.addNode(0.0, 0.0, True)
    n1 = network.addNode(1e-3, 2e-3, False)
    n2 = network.addNode(1e-3, 1e-3, False)
    n3 = network.addNode(1e-3, 0.0, False)
    n4 = network.addNode(2e-3, 2e-3, False)
    n5 = network.addNode(2e-3, 1e-3, False)
    n6 = network.addNode(2e-3, 0.0, False)
    n7 = network.addNode(3e-3, 1e-3, True)

    # Channels
    network.addPressurePump(n0, n1, 1e3)
    network.addPressurePump(n0, n2, 1e3)
    network.addPressurePump(n0, n3, 1e3)
    network.addRectangularChannel(n1, n4, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n2, n5, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n3, n6, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n4, n5, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n6, n5, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n5, n7, 1e-4, 1e-4, ChannelType.normal)

    network.sort()
    network.valid()

    # Define the simulation
    simulation = AbstractContinuousSimulation(network)

    # Fluid & Resistance Model
    f0 = simulation.addFluid(997, 1e-3, 1.0)
    simulation.setContinuousPhase(f0)
    simulation.setRectangularResistanceModel()

    simulation.simulate()

    simulation.saveResult("contAbst.JSON")

# Droplet Abstract
def abstractDroplet():

    # Define the network
    network = Network()

    # Nodes
    n0 = network.addNode(0.0, 0.0, False)
    n1 = network.addNode(1e-3, 0.0, False)
    n2 = network.addNode(2e-3, 0.0, False)
    n3 = network.addNode(2.5e-3, 0.86602540378e-3, False)
    n4 = network.addNode(3e-3, 0.0, False)
    n5 = network.addNode(4e-3, 0.0, True, True)

    # Channels
    c0 = network.addRectangularChannel(n0, n1, 1e-4, 3e-5, ChannelType.normal)
    network.addRectangularChannel(n1, n2, 1e-4, 3e-5, ChannelType.normal)
    network.addRectangularChannel(n2, n3, 1e-4, 3e-5, ChannelType.normal)
    network.addRectangularChannel(n2, n4, 1e-4, 3e-5, ChannelType.normal)
    network.addRectangularChannel(n3, n4, 1e-4, 3e-5, ChannelType.normal)
    network.addRectangularChannel(n4, n5, 1e-4, 3e-5, ChannelType.normal)
    network.addFlowRatePump(n5, n0, 3e-11)

    network.sort()
    network.valid()

    # Define the simulation
    simulation = AbstractDropletSimulation(network)

    # Fluid & Resistance Model
    water = simulation.addFluid(1e3, 1e-3, 1.0)
    oil = simulation.addFluid(1e3, 3e-3, 1.0)
    simulation.setContinuousPhase(water)
    simulation.setRectangularResistanceModel()
    d1 = simulation.addDroplet(oil, 4.5e-13)
    simulation.injectDroplet(d1, 0.0, c0, 0.5)

    simulation.simulate()

    simulation.saveResult("dropletAbst.JSON")

# Continuous Hybrid
def hybridContinuous():

    # Define the network
    network = Network()

    # Nodes
    n0 = network.addNode(0.0, 0.0, True)
    n1 = network.addNode(1e-3, 2e-3, False)
    n2 = network.addNode(1e-3, 1e-3, False)
    n3 = network.addNode(1e-3, 0.0, False)
    n4 = network.addNode(2e-3, 2e-3, False)
    n5 = network.addNode(1.75e-3, 1e-3, False)
    n6 = network.addNode(2e-3, 0.0, False)
    n7 = network.addNode(2e-3, 1.25e-3, False)
    n8 = network.addNode(2e-3, 0.75e-3, False)
    n9 = network.addNode(2.25e-3, 1e-3, False)
    n10 = network.addNode(3e-3, 1e-3, True)

    # Channels
    network.addRectangularChannel(n0, n1, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n0, n2, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n0, n3, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n1, n4, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n2, n5, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n3, n6, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n4, n7, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n6, n8, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n9, n10, 1e-4, 1e-4, ChannelType.normal)

    # Module
    network.addCfdModule("test", "../../../../examples/STL/cross.stl", [1.75e-3, 0.75e-3], [5e-4, 5e-4], [n5, n7, n8, n9], \
                      [[1.0, 0.0], [0.0, -1.0], [0.0, 1.0], [-1.0, 0.0]], [1e-4, 1e-4, 1e-4, 1e-4], \
                        1e-4, 1e-1, 0.1, 20, 1e-1, 0.55)

    # Pressure Pumps
    network.addPressurePump(n0, n1, 1e3)
    network.addPressurePump(n0, n2, 1e3)
    network.addPressurePump(n0, n3, 1e3)

    network.sort()
    network.valid()

    # Define the simulation
    simulation = HybridContinuousSimulation(network)

    # Fluid & Resistance Model
    f0 = simulation.addFluid(1e3, 1e-3, 1.0)
    simulation.setContinuousPhase(f0)
    simulation.setPoiseuilleResistanceModel()

    simulation.simulate()

    simulation.saveResult("contHybrid.JSON")

def abstractContinuousJSON():
    
    # Define the network
    network = Network()
    network.loadNetwork("../../../../examples/Abstract/Continuous/Network1.JSON")

    network.sort()
    network.valid()

    simulation = AbstractContinuousSimulation("../../../../examples/Abstract/Continuous/Network1.JSON", network)

    simulation.simulate()

    simulation.saveResult("contAbstjson.JSON")

def abstractDropletJSON():

    # Define the network
    network = Network()
    network.loadNetwork("../../../../examples/Abstract/Droplet/Network1.JSON")

    network.sort()
    network.valid()

    simulation = AbstractDropletSimulation("../../../../examples/Abstract/Droplet/Network1.JSON", network)

    simulation.simulate()

    simulation.saveResult("dropletAbstjson.JSON")

def hybridContinuousJSON():

    # Define the network
    network = Network()
    network.loadNetwork("../../../../examples/Hybrid/Network1a.JSON")

    simulation = HybridContinuousSimulation("../../../../examples/Hybrid/Network1a.JSON", network)

    network.valid()

    simulation.simulate()

    simulation.saveResult("contHybridjson.JSON")

def main():
    abstractContinuous()
    abstractContinuousJSON()
    abstractDroplet()
    abstractDropletJSON()
    # hybridContinuous()
    hybridContinuousJSON()

if __name__ == "__main__":
    main()


from mmft.simulator import *

# Continuous Abstract
def abstractContinuous():

    network = createNetwork()

    # Nodes
    n0 = network.addNode(0.0, 0.0, True)
    n1 = network.addNode(1e-3, 2e-3, False)
    n2 = network.addNode(1e-3, 1e-3, False)
    n3 = network.addNode(1e-3, 0.0, False)
    n4 = network.addNode(2e-3, 2e-3, False)
    n5 = network.addNode(2e-3, 1e-3, False)
    n6 = network.addNode(2e-3, 0.0, False)
    n7 = network.addNode(3e-3, 1e-3, True)

    # Channels
    network.addPressurePump(n0, n1, 1e3)
    network.addPressurePump(n0, n2, 1e3)
    network.addPressurePump(n0, n3, 1e3)
    network.addRectangularChannel(n1, n4, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n2, n5, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n3, n6, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n4, n5, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n6, n5, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n5, n7, 1e-4, 1e-4, ChannelType.normal)

    # Simulation
    simulation = AbstractContinuous(network)

    # Fluid & Resistance Model
    f0 = simulation.addFluid(1e-3, 997)
    simulation.setContinuousPhase(f0)
    simulation.set1DResistanceModel()

    simulation.simulate()

    # Results
    results = simulation.getResults()
    results.printLastState()

    # simulation.saveResult("continuousAbstract.JSON")

def main():
    abstractContinuous()

if __name__ == "__main__":
    main()


from mmft.simulator import *

# Droplet Abstract
def abstractDroplet():

    network = createNetwork()

    # Nodes
    n0 = network.addNode(0.0, 0.0, False)
    n1 = network.addNode(1e-3, 0.0, False)
    n2 = network.addNode(2e-3, 0.0, False)
    n3 = network.addNode(2.5e-3, 0.86602540378e-3, False)
    n4 = network.addNode(3e-3, 0.0, False)
    n5 = network.addNode(4e-3, 0.0, True, True)

    # Channels
    c0 = network.addRectangularChannel(n0, n1, 1e-4, 3e-5, ChannelType.normal)
    network.addRectangularChannel(n1, n2, 1e-4, 3e-5, ChannelType.normal)
    network.addRectangularChannel(n2, n3, 1e-4, 3e-5, ChannelType.normal)
    network.addRectangularChannel(n2, n4, 1e-4, 3e-5, ChannelType.normal)
    network.addRectangularChannel(n3, n4, 1e-4, 3e-5, ChannelType.normal)
    network.addRectangularChannel(n4, n5, 1e-4, 3e-5, ChannelType.normal)
    network.addFlowRatePump(n5, n0, 3e-11)

    # Simulation
    simulation = AbstractDroplet(network)

    # Fluid & Resistance Model
    water = simulation.addFluid(1e-3, 1e3)
    oil = simulation.addFluid(3e-3, 1e3)
    simulation.setContinuousPhase(water)
    simulation.set1DResistanceModel()
    d1 = simulation.addDroplet(oil, 4.5e-13)
    simulation.addDropletInjection(d1, 0.0, c0, 0.5)

    simulation.simulate()

    # Results
    result = simulation.getResults()
    result.printLastState()

def main():
    abstractDroplet()

if __name__ == "__main__":
    main()


from mmft.simulator import *

# Continuous CFD
def cfdContinuous():

    network = createNetwork()

    # Nodes
    n1 = network.addNode(1e-3, 2e-3, True)
    n2 = network.addNode(1e-3, 1e-3, True)
    n3 = network.addNode(1e-3, 0.0, True)
    n4 = network.addNode(2e-3, 2e-3, False)
    n5 = network.addNode(2e-3, 1e-3, False)
    n6 = network.addNode(2e-3, 0.0, False)
    n7 = network.addNode(3e-3, 1e-3, True)

    # Channels
    network.addRectangularChannel(n1, n4, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n2, n5, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n3, n6, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n4, n5, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n6, n5, 1e-4, 1e-4, ChannelType.normal)
    network.addRectangularChannel(n5, n7, 1e-4, 1e-4, ChannelType.normal)

    # Simulation
    simulation = CfdContinuous(network)
    simulation.setResolution(40)

    # Fluid & Resistance Model
    f0 = simulation.addFluid(1e-3, 1e3)
    simulation.setContinuousPhase(f0)

    # species
    s1 = simulation.addSpecie(1e-6, 1.0, 1.0)

    # Define Boundary Conditions
    simulation.addPressureBC(n1, 1e2)
    simulation.addPressureBC(n2, 1e2)
    simulation.addPressureBC(n3, 1e2)
    simulation.addPressureBC(n7, 0.0)

    simulation.addConcentrationBC(n1, s1, 1.2)
    simulation.addConcentrationBC(n2, s1, 1.2)
    simulation.addConcentrationBC(n3, s1, 1.0)

    simulation.simulate()

def main():
    cfdContinuous()

if __name__ == "__main__":
    main()


from __future__ import annotations

import os
import pickle
import sys

import pytest


@pytest.mark.skipif(
    sys.platform.startswith("emscripten"), reason="Requires loadable modules"
)
def test_independent_subinterpreters():
    """Makes sure the internals object differs across independent subinterpreters"""

    sys.path.append(".")

    # This is supposed to be added in 3.14.0b3
    if sys.version_info >= (3, 15):
        import interpreters
    elif sys.version_info >= (3, 13):
        import _interpreters as interpreters
    elif sys.version_info >= (3, 12):
        import _xxsubinterpreters as interpreters
    else:
        pytest.skip("Test requires the interpreters stdlib module")

    m = pytest.importorskip("mod_per_interpreter_gil")

    if not m.defined_PYBIND11_HAS_SUBINTERPRETER_SUPPORT:
        pytest.skip("Does not have subinterpreter support compiled in")

    code = """
import mod_per_interpreter_gil as m
import pickle
with open(pipeo, 'wb') as f:
    pickle.dump(m.internals_at(), f)
"""

    interp1 = interpreters.create()
    interp2 = interpreters.create()
    try:
        try:
            res0 = interpreters.run_string(interp1, "import mod_shared_interpreter_gil")
            if res0 is not None:
                res0 = res0.msg
        except Exception as e:
            res0 = str(e)

        pipei, pipeo = os.pipe()
        interpreters.run_string(interp1, code, shared={"pipeo": pipeo})
        with open(pipei, "rb") as f:
            res1 = pickle.load(f)

        pipei, pipeo = os.pipe()
        interpreters.run_string(interp2, code, shared={"pipeo": pipeo})
        with open(pipei, "rb") as f:
            res2 = pickle.load(f)

        # do this while the two interpreters are active
        import mod_per_interpreter_gil as m2

        assert m.internals_at() == m2.internals_at(), (
            "internals should be the same within the main interpreter"
        )
    finally:
        interpreters.destroy(interp1)
        interpreters.destroy(interp2)

    assert "does not support loading in subinterpreters" in res0, (
        "cannot use shared_gil in a default subinterpreter"
    )
    assert res1 != m.internals_at(), "internals should differ from main interpreter"
    assert res2 != m.internals_at(), "internals should differ from main interpreter"
    assert res1 != res2, "internals should differ between interpreters"

    # do this after the two interpreters are destroyed and only one remains
    import mod_per_interpreter_gil as m3

    assert m.internals_at() == m3.internals_at(), (
        "internals should be the same within the main interpreter"
    )


@pytest.mark.skipif(
    sys.platform.startswith("emscripten"), reason="Requires loadable modules"
)
def test_dependent_subinterpreters():
    """Makes sure the internals object differs across subinterpreters"""

    sys.path.append(".")

    if sys.version_info >= (3, 15):
        import interpreters
    elif sys.version_info >= (3, 13):
        import _interpreters as interpreters
    elif sys.version_info >= (3, 12):
        import _xxsubinterpreters as interpreters
    else:
        pytest.skip("Test requires the interpreters stdlib module")

    m = pytest.importorskip("mod_shared_interpreter_gil")

    if not m.defined_PYBIND11_HAS_SUBINTERPRETER_SUPPORT:
        pytest.skip("Does not have subinterpreter support compiled in")

    code = """
import mod_shared_interpreter_gil as m
import pickle
with open(pipeo, 'wb') as f:
    pickle.dump(m.internals_at(), f)
"""

    try:
        interp1 = interpreters.create("legacy")
    except TypeError:
        pytest.skip("interpreters module needs to support legacy config")

    try:
        pipei, pipeo = os.pipe()
        interpreters.run_string(interp1, code, shared={"pipeo": pipeo})
        with open(pipei, "rb") as f:
            res1 = pickle.load(f)

        # do this while the other interpreter is active
        import mod_shared_interpreter_gil as m2

        assert m.internals_at() == m2.internals_at(), (
            "internals should be the same within the main interpreter"
        )
    finally:
        interpreters.destroy(interp1)

    assert res1 != m.internals_at(), "internals should differ from main interpreter"

    # do this after the other interpreters are destroyed and only one remains
    import mod_shared_interpreter_gil as m3

    assert m.internals_at() == m3.internals_at(), (
        "internals should be the same within the main interpreter"
    )



"""MMFT Simulation Package"""

from mmft.simulator.pysimulator import (
    AbstractConcentration,
    AbstractContinuous,
    AbstractDroplet,
    AbstractMembrane,
    CfdConcentration,
    CfdContinuous,
    CfdModule,
    CFDSimulator,
    Channel,
    ChannelType,
    ConcentrationSemantics,
    createNetwork,
    CylindricalChannel,
    DiffusiveMixture,
    Droplet,
    DropletInjection,
    Edge,
    FlowRatePump,
    Fluid,
    HybridConcentration,
    HybridContinuous,
    lbmSimulator,
    Membrane,
    Mixture,
    MixtureInjection,
    Module,
    ModuleType,
    Network,
    networkFromJSON,
    Node,
    Opening,
    Platform,
    PressurePump,
    RectangularChannel,
    Simulation,
    Specie,
    State,
    SimulationResult,
    Tank,
    Type
)

__all__ = [
    'AbstractConcentration',
    'AbstractContinuous',
    'AbstractDroplet',
    'AbstractMembrane',
    'CfdConcentration',
    'CfdContinuous',
    'CfdModule',
    'CFDSimulator',
    'Channel',
    'ChannelType',
    'ConcentrationSemantics',
    'createNetwork',
    'CylindricalChannel',
    'DiffusiveMixture',
    'Droplet',
    'DropletInjection',
    'Edge',
    'FlowRatePump',
    'Fluid',
    'HybridConcentration',
    'HybridContinuous',
    'lbmSimulator',
    'Membrane',
    'Mixture',
    'MixtureInjection',
    'Module',
    'ModuleType',
    'Network',
    'networkFromJSON',
    'Node',
    'Opening',
    'Platform',
    'PressurePump',
    'RectangularChannel',
    'Simulation',
    'SimulationResult',
    'Specie',
    'State',
    'Tank',
    'Type'
]
